
function Marker(poiData) {

    this.poiData = poiData;
    this.ready = false;
    this.isSelected = false;

    this.isHidden = false;
    this.clickReady = false;

    /* Create the AR.GeoLocation from the poi data. */
    this.markerLocation = new AR.GeoLocation(poiData.latitude, poiData.longitude, AR.CONST.UNKNOWN_ALTITUDE);

    var offset = poiData.offsetY;

    this.drawable = null;
    if (poiData.type === 'houses') {
        this.drawable = World.markerDrawableHouse;
    }
    if (poiData.type === 'museums') {
        this.drawable = World.markerDrawableMuseum;
    }
    if (poiData.type === 'monuments') {
        this.drawable = World.markerDrawableMonument;
    }
    if (poiData.type === 'places') {
        this.drawable = World.markerDrawablePlace;
    }
    if (poiData.type === 'quests') {
        this.drawable = World.markerDrawableQuest;
    }
    if (poiData.type === 'routes') {
        this.drawable = World.markerDrawableRoute;
    }
    if (poiData.type === 'personalities') {
        this.drawable = World.markerDrawablePersonality;
    }

    this.markerDrawableIdle = new AR.ImageDrawable(this.drawable, 4, {
        zOrder: 0,
        opacity: 0.0,
        offsetY: offset,
        offsetX: poiData.offsetX,
        onClick: Marker.prototype.getOnClickTrigger(this)
    });

    this.titleLabel = new AR.Label(poiData.title.trunc2(18), 0.6, {
        opacity: 0.0,
        zOrder: 1,
        offsetY: offset + 0.5,
        offsetX: poiData.offsetX - 2.2,
        verticalAnchor: AR.CONST.VERTICAL_ANCHOR.TOP,
        horizontalAnchor: AR.CONST.HORIZONTAL_ANCHOR.LEFT,
        translate: {
            y: 0.55
        },
        style: {
            textColor: '#FFFFFF',
            fontStyle: AR.CONST.FONT_STYLE.BOLD
        },
        onClick: Marker.prototype.getOnClickTrigger(this)
    });

    this.titleLabel2 = new AR.Label(poiData.title.trunc3(18), 0.6, {
        opacity: 0.0,
        zOrder: 1,
        offsetY: offset,
        offsetX: poiData.offsetX - 2.2,
        verticalAnchor: AR.CONST.VERTICAL_ANCHOR.TOP,
        horizontalAnchor: AR.CONST.HORIZONTAL_ANCHOR.LEFT,
        translate: {
            y: 0.55
        },
        style: {
            textColor: '#FFFFFF',
            fontStyle: AR.CONST.FONT_STYLE.BOLD
        },
        onClick: Marker.prototype.getOnClickTrigger(this)
    });

    this.markerObject = new AR.GeoObject(this.markerLocation, {
        drawables: {
            cam: [this.markerDrawableIdle, this.titleLabel, this.titleLabel2]
        }
    });

    return this.updateDistance(this, poiData);
}


Marker.prototype.updateDistance = function (marker, poiData){
    marker.poiData = poiData;
    var offset = poiData.offsetY;

    var distance = poiData.distance * 1000

    if (marker.isSelected === true){
        return marker;
    }

    marker.isHidden = distance > 200;
    marker.clickReady = distance <= 150;

    if (marker.ready === false || (marker.ready === true && distance < 150)) {
        if (marker.ready === true) {
            marker.markerObject.drawables.removeCamDrawable(marker.descriptionLabelTop);
            marker.descriptionLabelTop.destroy();
        }
        marker.descriptionLabelTop = new AR.Label(distance + ' м', 0.6, {
            opacity: 0.0,
            zOrder: 1,
            offsetY: offset + 1.3,
            offsetX: poiData.offsetX + 2.2,
            verticalAnchor: AR.CONST.VERTICAL_ANCHOR.TOP,
            horizontalAnchor: AR.CONST.HORIZONTAL_ANCHOR.RIGHT,
            translate: {
                y: -0.55
            },
            style: {
                textColor: '#FFFFFF'
            }
        });
        marker.markerObject.drawables.addCamDrawable(marker.descriptionLabelTop);
    }

    if (distance > 200) {
        marker.titleLabel.opacity = 0.0;
        marker.titleLabel2.opacity = 0.0;
        marker.markerDrawableIdle.opacity = 0.0;

        marker.descriptionLabelTop.opacity = 0.0;
    } else if (distance > 150 && distance < 200) {
        marker.titleLabel.opacity = 0.3;
        marker.titleLabel2.opacity = 0.3;
        marker.markerDrawableIdle.opacity = 0.3;

        marker.descriptionLabelTop.opacity = 0.3;
    } else if (distance > 100 && distance < 150) {
        marker.titleLabel.opacity = 0.5;
        marker.titleLabel2.opacity = 0.5;
        marker.markerDrawableIdle.opacity = 0.5;

        marker.descriptionLabelTop.opacity = 0.5;
    } else {
        marker.titleLabel.opacity = 1.0;
        marker.titleLabel2.opacity = 1.0;
        marker.markerDrawableIdle.opacity = 1.0;

        marker.descriptionLabelTop.opacity = 1;
    }
    this.ready = true;
    return marker;
};

Marker.prototype.destroy = function destroyFn(marker){
    try{
        marker.titleLabel.destroy();
        marker.titleLabel2.destroy();
        marker.markerDrawableIdle.destroy();
        marker.markerLocation.destroy();
        marker.markerObject.destroy();
        marker.descriptionLabelTop.destroy();
    }catch (e) {
        //alert(e);
    }
};

Marker.prototype.getOnClickTrigger = function(marker) {
    return function() {
        try {
            World.onMarkerSelected(marker);
        } catch (err) {
            //alert(err);
        }
        return true;
    };
};

Marker.prototype.setSelected = function(marker) {

    marker.isSelected = true;

    if (marker.isHidden && marker.clickReady === false){
        marker.titleLabel.opacity = 1.0;
        marker.titleLabel2.opacity = 1.0;
        marker.markerDrawableIdle.opacity = 1.0;
        marker.descriptionLabelTop.opacity = 0.0;
        marker.clickReady = true;
        return;
    }
    if (marker.clickReady) {
        try{
            AR.platform.sendJSONObject(marker);
        }catch (e) {

        }
    }
};

Marker.prototype.setDeselected = function(marker) {
    marker.isSelected = false;
    marker.updateDistance(marker, marker.poiData);
};

String.prototype.trunc = function(n) {
    return this.substr(0, n - 1) + (this.length > n ? '...' : '');
};
String.prototype.trunc2 = function (n) {
    var i = this.indexOf(" ");
    if (i === -1) return this;
    var res = "";
    while (i < n) {
        res = this.substr(0, i);
        i = this.indexOf(" ", i + 1);
        if (i === -1) return res;
    }
    return res;
};
String.prototype.trunc3 = function (n) {
    var l = this.trunc2(n).length;
    if (l === this.length)
        return "";
    return this.substr(l + 1, this.length - 1 - l).trunc(n-2);
};

