/* Implementation of AR-Experience (aka "World"). */
var World = {
    /* True once data was fetched. */
    initiallyLoadedData: false,

    /* Different POI-Marker assets. */
    markerDrawableHouse: null,
    markerDrawableMuseum: null,
    markerDrawableMonument: null,
    markerDrawablePlace: null,
    markerDrawableQuest: null,
    markerDrawableRoute: null,
    markerDrawablePersonality: null,

    /* List of AR.GeoObjects that are currently shown in the scene / World. */
    markerList: [],
    markerPathList: [],

    /* the last selected marker. */
    currentMarker: null,

    loadPoisFromJsonData: function loadPoisFromJsonDataFn(poiData) {
        if (Array.isArray(World.markerList) && World.markerList.length > 0){
            World.markerList.forEach(function(element) {
                element.destroy(element);
            });
            World.markerList.splice(0, World.markerList.length);
        }

        World.markerDrawableHouse = new AR.ImageResource("assets/houses.png", {
            onError: World.onError
        });
        World.markerDrawableMonument = new AR.ImageResource("assets/monuments.png", {
            onError: World.onError
        });
        World.markerDrawableMuseum = new AR.ImageResource("assets/museums.png", {
            onError: World.onError
        });
        World.markerDrawablePersonality = new AR.ImageResource("assets/personalities.png", {
            onError: World.onError
        });
        World.markerDrawablePlace = new AR.ImageResource("assets/places.png", {
            onError: World.onError
        });
        World.markerDrawableRoute = new AR.ImageResource("assets/routes.png", {
            onError: World.onError
        });
        World.markerDrawableQuest = new AR.ImageResource("assets/quest.png", {
            onError: World.onError
        });

        for (var currentPlaceNr = 0; currentPlaceNr < poiData.length; currentPlaceNr++) {
            var singlePoi = {
                "id": poiData[currentPlaceNr].id,
                "latitude": parseFloat(poiData[currentPlaceNr].latitude),
                "longitude": parseFloat(poiData[currentPlaceNr].longitude),
                "altitude": parseFloat(poiData[currentPlaceNr].altitude),
                "title": poiData[currentPlaceNr].name,
                "type": poiData[currentPlaceNr].type,
                "distance": poiData[currentPlaceNr].distance,
                "distanceNumber": poiData[currentPlaceNr].distanceNumber,
                "offsetX": poiData[currentPlaceNr].offsetX,
                "offsetY": poiData[currentPlaceNr].offsetY
            };

            World.markerList.push(new Marker(singlePoi));
        }
    },

    loadPathPoisFromJsonData: function loadPathPoisFromJsonDataFn(poiData) {
        if (Array.isArray(World.markerPathList) && World.markerPathList.length > 0){
            World.markerPathList.forEach(function(element) {
                element.destroy(element);
            });
            World.markerPathList.splice(0, World.markerPathList.length);
        }

        World.markerDrawablePath = new AR.ImageResource("assets/pin-group@3x.png", {
            onError: World.onError
        });

        for (var currentPlaceNr = 0; currentPlaceNr < poiData.length; currentPlaceNr++) {
            var singlePoi = {
                "id": poiData[currentPlaceNr].id,
                "latitude": parseFloat(poiData[currentPlaceNr].latitude),
                "longitude": parseFloat(poiData[currentPlaceNr].longitude),
                "altitude": parseFloat(poiData[currentPlaceNr].altitude),
                "title": poiData[currentPlaceNr].name,
                "type": poiData[currentPlaceNr].type,
                "distance": poiData[currentPlaceNr].distance,
                "distanceNumber":poiData[currentPlaceNr].distanceNumber,
                "rotationX":poiData[currentPlaceNr].rotationX
            };

            World.markerPathList.push(new MarkerRoute(singlePoi));
        }
    },

    onMarkerSelected: function onMarkerSelectedFn(marker) {
        try{
            if (World.currentMarker) {
                if (World.currentMarker.poiData.id !== marker.poiData.id) {
                    World.currentMarker.setDeselected(World.currentMarker);
                }
            }
        }catch (e) {
            //alert(e);
        }

        try{
            marker.setSelected(marker);
            World.currentMarker = marker;
        }catch (e) {

        }
    },

    onScreenClick: function onScreenClickFn() {
        try{
            if (World.currentMarker) {
                World.currentMarker.setDeselected(World.currentMarker);
            }
        }catch (e) {

        }
        World.currentMarker = false;
    },

    onLocationChanged: function locationChangedFn(lat, lon, alt, acc) {
        World.markerList.forEach(function(element) {
            var poiData = element.poiData;
            poiData.distance = Math.floor(element.markerObject.locations[0].distanceToUser()) / 1000;
            element.updateDistance(element, poiData);
        });
    },

    onError: function onErrorFn(error) {
        //alert(error);
    }
};

AR.context.onScreenClick = World.onScreenClick;

AR.context.onLocationChanged = World.onLocationChanged;
